@@ -14,7 +14,7 @@
 ** Most of the code in this file may be omitted by defining the
 ** SQLITE_OMIT_VACUUM macro.
 **
-** $Id: vacuum.c,v 1.7 2004/07/21 20:50:45 matt Exp $
+** $Id: vacuum.c,v 1.8 2004/08/09 13:08:31 matt Exp $
 */
 #include "sqliteInt.h"
 #include "os.h"
@@ -160,11 +160,17 @@ int sqlite3RunVacuum(char **pzErrMsg, sqlite *db){
   ** in the temporary database.
   */
   rc = execExecSql(db, 
-      "SELECT 'CREATE ' || type || ' vacuum_db.' || "
-      "substr(sql, length(type)+9, 1000000) "
-      "FROM sqlite_master "
-      "WHERE type != 'trigger' AND sql IS NOT NULL "
-      "ORDER BY (type != 'table');" 
+      "SELECT 'CREATE TABLE vacuum_db.' || substr(sql,14,100000000) "
+      "  FROM sqlite_master WHERE type='table' "
+      "UNION ALL "
+      "SELECT 'CREATE INDEX vacuum_db.' || substr(sql,14,100000000) "
+      "  FROM sqlite_master WHERE sql LIKE 'CREATE INDEX %' "
+      "UNION ALL "
+      "SELECT 'CREATE UNIQUE INDEX vacuum_db.' || substr(sql,21,100000000) "
+      "  FROM sqlite_master WHERE sql LIKE 'CREATE UNIQUE INDEX %'"
+      "UNION ALL "
+      "SELECT 'CREATE VIEW vacuum_db.' || substr(sql,13,100000000) "
+      "  FROM sqlite_master WHERE type='view'"
   );
   if( rc!=SQLITE_OK ) goto end_of_vacuum;
 
@@ -186,10 +192,8 @@ int sqlite3RunVacuum(char **pzErrMsg, sqlite *db){
   ** point also.
   */
   rc = execExecSql(db, 
-      "SELECT 'CREATE ' || type || ' vacuum_db.' || "
-      "substr(sql, length(type)+9, 1000000) "
-      "FROM sqlite_master "
-      "WHERE type = 'trigger' AND sql IS NOT NULL;"
+      "SELECT 'CREATE TRIGGER  vacuum_db.' || substr(sql, 16, 1000000) "
+      "FROM sqlite_master WHERE type='trigger'"
   );
   if( rc!=SQLITE_OK ) goto end_of_vacuum;
 
@@ -247,6 +251,7 @@ end_of_vacuum:
     sqliteFree(zTemp);
   }
   if( zSql ) sqliteFree( zSql );
+  sqlite3ResetInternalSchema(db, 0);
 #endif
   return rc;
-} 
+}
